open Arbres_naifs
open Graphics

type 'a dxt = Vide | Noeud of ('a dxt * 'a t * (int * int) list * 'a dxt)

let rec calculer_dx arbre =
  try
    match (calculer_dx (gauche arbre), calculer_dx (droite arbre)) with
      | Vide, Vide -> Noeud (Vide, arbre, [], Vide)
      | Vide, (Noeud (_,_,dv,_) as d) ->
	  let nouv = List.map (fun (x,y) -> (x+1,y+1)) dv in
	  Noeud (Vide, arbre, (1,1)::nouv, d)
      | (Noeud (_,_,gv,_) as g), Vide ->
	  let nouv = List.map (fun (x,y) -> (x-1,y-1)) gv in
	  Noeud (g, arbre, (-1,-1)::nouv, Vide)
      | (Noeud (_,_,gv,_) as g), (Noeud (_,_,dv,_) as d) ->
	  let rec separation deja = function
	    | ([], _) | (_, []) -> deja
	    | ((_,x)::q, (y,_)::r) -> separation (max deja ((x-y+3)/2)) (q,r)
	  in
	  let dx = separation 1 (gv, dv) in
	  let rec recombine = function
	    | ([], []) -> []
	    | ([], r) -> List.map (fun (x,y) -> (x+dx,y+dx)) r
	    | (r, [] ) -> List.map (fun (x,y) -> (x-dx,y-dx)) r
	    | ((x1,y1)::q, (x2,y2)::r) -> (x1-dx,y2+dx) :: (recombine (q,r))
	  in
	  Noeud (g, arbre, (-dx,dx) :: (recombine (gv, dv)), d)
  with Not_found -> Vide

let dessiner dy rad x y arbre =
  let rec aux x y = function
    | Vide -> ()
    | Noeud (g, v, dxlist, d) ->
	let (dx1,dx2) = match dxlist with
	  | (dx1,dx2) :: _ -> (dx1*rad*2, dx2*rad*2)
	  | _ -> (0, 0)
	in
	set_color black;
	if g <> Vide then draw_segments [| (x,y,x+dx1,y-dy) |];
	if d <> Vide then draw_segments [| (x,y,x+dx2,y-dy) |];
	begin try
	  let clef = clef v in
	  let (fcolor, bcolor) = rechercher clef v in
	  set_color bcolor;
	  fill_circle x y rad;
	  set_color black;
	  draw_circle x y rad;
	  set_color fcolor;
	  let str = Printf.sprintf "%d" clef in
	  let (w,h) = text_size str in
	  moveto (x-w/2) (y-h/2);
	  draw_string str;
	with
	    Not_found -> ()
	end;
	aux (x+dx1) (y-dy) g;
	aux (x+dx2) (y-dy) d
  in aux x y (calculer_dx arbre)

let r = (black, white)
and n = (white, black)

let construire liste = List.fold_left (fun a (x,c) -> inserer x c a) vide liste

let arbre1a = construire [18, r]
let arbre1b = construire [18, n]
and arbre2  = construire [18, n; 98, r]
and arbre3  = construire [18, n; 98, r; 51, r]
and arbre4  = construire [18, n; 51, r; 98, r]
and arbre5  = construire [51, n; 18, r; 98, r]
and arbre6a = construire [51, n; 18, r; 98, r; 70, r]
and arbre6b = construire [51, r; 18, n; 98, n; 70, r]
and arbre6c = construire [51, n; 18, n; 98, n; 70, r]
and arbre7  = construire [51, n; 18, n; 98, n; 70, r; 62, r]
and arbre8  = construire [51, n; 18, n; 70, n; 62, r; 98, r] ;;

let fleche x y dx dy =
  draw_segments [| (x, y, x+dx, y+dy); (x, y, x+dx, y-dy) |] ;;
  

open_graph " 590x580" ;;
set_font "Courier-Bold" ;;
set_text_size 16 ;;

dessiner 60 16  94 560 arbre1a ;;
dessiner 60 16 230 560 arbre1b ;;
dessiner 60 16 360 560 arbre2  ;;
dessiner 60 16 490 560 arbre3  ;;
dessiner 60 16 490 380 arbre4  ;;
dessiner 60 16 360 380 arbre5  ;;
dessiner 60 16 230 380 arbre6a ;;
dessiner 60 16  94 380 arbre6b ;;
dessiner 60 16  94 200 arbre6c ;;
dessiner 60 16 230 200 arbre7  ;;
dessiner 60 16 360 200 arbre8  ;;


draw_segments
  [| (124, 560, 200, 560);
     (260, 560, 330, 560);
     (390, 560, 460, 560);
     (124, 380, 200, 380);
     (260, 380, 330, 380);
     (390, 380, 460, 380);
     (124, 200, 200, 200);
     (260, 200, 330, 200);
     (520, 560, 580, 560);
     (580, 560, 580, 380);
     (580, 380, 520, 380);
     ( 10, 380,  64, 380);
     ( 10, 380,  10, 200);
     ( 64, 200,  10, 200)
  |] ;;

fleche 200 560 (-8) 5 ;;
fleche 330 560 (-8) 5 ;;
fleche 460 560 (-8) 5 ;;
fleche 124 380 ( 8) 5 ;;
fleche 260 380 ( 8) 5 ;;
fleche 390 380 ( 8) 5 ;;
fleche 200 200 (-8) 5 ;;
fleche 330 200 (-8) 5 ;;
fleche 520 380 ( 8) 5 ;;
fleche  64 200 (-8) 5 ;;
