(**********************************************************************
 * ARBRES NAIFS
 * par Stphane GLONDU (Avril 2004)
 **********************************************************************)

(** Dfinition des types **********************************************)

type clef = int
	
type 'a t =
  | Vide
  | Noeud of ('a t * clef * 'a * 'a t)

(** Fonctions pour modules externes ***********************************)

let vide = Vide
and gauche = function
  | Noeud (g,_,_,_) -> g
  | _ -> raise Not_found
and droite = function
  | Noeud (_,_,_,d) -> d
  | _ -> raise Not_found
and clef = function
  | Noeud (_,x,_,_) -> x
  | _ -> raise Not_found
and dcolor c _ = c.(0)

(** Vrification et recherche *****************************************)

let verifier arbre =
  let rec aux inf sup h = function
    | Vide -> (0, h)
    | Noeud (g,x,_,d) ->
        if inf < x && x < sup then begin
          let (ng, hg) = aux inf x h g in
          let (nd, hd) = aux x sup h d in
          (1+ng+nd, 1+(max hg hd))
        end else invalid_arg "arbre incorrect"
  in aux min_int max_int (-1) arbre

let rec rechercher x = function
  | Vide -> raise Not_found
  | Noeud (g,r,_,_) when x < r -> rechercher x g
  | Noeud (_,r,_,d) when r < x -> rechercher x d
  | Noeud (_,_,v,_) -> v

(** Insertion *********************************************************)

let rec inserer nx nv = function
  | Vide -> Noeud (Vide, nx, nv, Vide)
  | Noeud (g,x,v,d) when nx < x -> Noeud (inserer nx nv g, x, v, d)
  | Noeud (g,x,v,d) when x < nx -> Noeud (g, x, v, inserer nx nv d)
  | Noeud (g,x,v,d) -> Noeud (g, x, nv, d)

(** Suppression *******************************************************)

let rec minimum = function
  | Vide -> raise Not_found
  | Noeud (Vide, x, d, _) -> (x, d)
  | Noeud (g,_,_,_) -> minimum g

let rec supprimer_minimum = function
  | Vide -> invalid_arg "supprimer_minimum"
  | Noeud (Vide, _, _, d) -> d
  | Noeud (g,x,v,d) -> Noeud (supprimer_minimum g, x, v, d)

let fusionner = function
  | (Vide, a) | (a, Vide) -> a
  | (g, d) ->
      let (x,v) = minimum d in
      Noeud (g, x, v, supprimer_minimum d)

let rec supprimer nx = function
  | Vide -> Vide
  | Noeud (g,x,v,d) when nx < x -> Noeud (supprimer nx g, x, v, d)
  | Noeud (g,x,v,d) when x < nx -> Noeud (g, x, v, supprimer nx d)
  | Noeud (g,_,_,d) -> fusionner (g, d)

(**********************************************************************)
