(**********************************************************************
 * ARBRES ROUGE-NOIR
 * par Stphane GLONDU (Avril 2004)
 **********************************************************************)

(** Dfinition des types **********************************************)


type clef = int
type couleur = Rouge | Noir | NoirNoir
	
type 'a t =
  | Vide | VideN
  | Noeud of ('a t * couleur * clef * 'a * 'a t)

let couleur = function
  | Vide        -> Noir
  | VideN       -> NoirNoir
  | @(_ _[c] _) -> c

(** Fonctions pour modules externes ***********************************)

let vide = Vide
and gauche = function @(g _ _) -> g  | _ -> raise Not_found
and droite = function @(_ _ d) -> d  | _ -> raise Not_found
and clef    = function @(_ r _) -> rx | _ -> raise Not_found
and dcolor c = function
  | @(_ _[Rouge] _) -> c.(1)
  | @(_ _[Noir ] _) -> c.(2)
  | _               -> raise Not_found

(** Vrification et recherche *****************************************)

let verifier arbre =
  let error () = invalid_arg "arbre rouge-noir incorrect" in
  if couleur arbre <> Noir then error ();
  let hn_def = ref (-1) in
  let rec aux inf sup h hn = function
    | VideN | @(_ _ [NoirNoir] _) -> error ()
    | Vide ->
	if !hn_def = -1 then (hn_def := hn; 0, h)
	else (if !hn_def = hn then 0, h else error ())
    | @(g r d) ->
	if inf < rx && rx < sup then begin
	  let hn' = hn + (if rc = Noir then 1 else 0) in
	  let (ng, hg) = aux inf rx h hn' g in
          let (nd, hd) = aux rx sup h hn' d in
	  (1+ng+nd, 1+(max hg hd))
	end else error ()
  in aux min_int max_int (-1) 0 arbre

let rec rechercher x = function
  | Vide                 -> raise Not_found
  | VideN                -> invalid_arg "arbre rouge-noir incorrect"
  | @(g r _) when x < rx -> rechercher x g
  | @(_ r d) when rx < x -> rechercher x d
  | @(_ r _)             -> rv

(** Insertion *********************************************************)

let cor_ins_g = function
  | @((a b[Rouge] c) d (e f[Rouge] g)) when couleur a = Rouge || couleur c = Rouge ->
    @((a b[Noir ] c) d[Rouge] (e f[Noir ] g))
  | @((a b[Rouge] (c d[Rouge] e)) f g) when couleur g = Noir ->
    @((a b[Rouge] c) d[Noir ] (e f[Rouge] g))
  | @(((a b[Rouge] c) d[Rouge] e) f g) when couleur g = Noir ->
    @((a b[Rouge] c) d[Noir ] (e f[Rouge] g))
  | a -> a

let cor_ins_d = function
  | @((a b[Rouge] c) d (e f[Rouge] g)) when couleur e = Rouge || couleur g = Rouge ->
    @((a b[Noir ] c) d[Rouge] (e f[Noir ] g))
  | @(a b ((c d[Rouge] e) f[Rouge] g)) when couleur a = Noir ->
    @((a b[Rouge] c) d[Noir ] (e f[Rouge] g))
  | @(a b (c d[Rouge] (e f[Rouge] g))) when couleur a = Noir ->
    @((a b[Rouge] c) d[Noir ] (e f[Rouge] g))
  | a -> a

let inserer nx nv a =
  let rec aux = function
    | VideN                 -> failwith "inserer"
    | Vide                  -> @({Vide} n[Rouge] {Vide})
    | @(g r d) when nx < rx -> cor_ins_g @({aux g} r d)
    | @(g r d) when rx < nx -> cor_ins_d @(g r {aux d})
    | @(g _[c] d)           -> @(g n[c] d)
  in match aux a with
    | @(g r d) -> @(g r[Noir] d)
    | _        -> failwith "inserer"

(** Suppression *******************************************************)

let eclaircir = function
  | @(g r[NoirNoir] d) -> @(g r[Noir] d)
  | VideN              -> Vide
  | _                  -> invalid_arg "eclaircir"

let cor_sup_g =
  let aux = function
    | @(a b (c d[Noir] e)) when couleur c = Noir && couleur e = Noir ->
      @(a b[if bc = Rouge then Noir else NoirNoir] (c d[Rouge] e))
    | @(a b ((c d[Rouge] e) f[Noir ] g))       when couleur g = Noir ->
      @((a b[Noir] c) d[bc] (e f[Noir] g))
    | @(a b (c d[Noir ] (e f[Rouge] g)))                             ->
      @((a b[Noir] c) d[bc] (e f[Noir] g))
    | _ -> failwith "cor_sup_g"
  in function
    | @(a b (c d[Rouge] e)) when couleur a = NoirNoir ->
      @({aux @({eclaircir a} b[Rouge] c)} d[Noir] e)
    | @(a b c)              when couleur a = NoirNoir -> aux @({eclaircir a} b c)
    | a -> a

let cor_sup_d =
  let aux = function
    | @((a b[Noir] c) d e) when couleur a = Noir && couleur c = Noir ->
      @((a b[Rouge] c) d[if dc = Rouge then Noir else NoirNoir] e)
    | @((a b[Noir ] (c d[Rouge] e)) f g)       when couleur a = Noir ->
      @((a b[Noir] c) d[fc] (e f[Noir] g))
    | @(((a b[Rouge] c) d[Noir ] e) f g)                             ->
      @((a b[Noir] c) d[fc] (e f[Noir] g))
    | _ -> failwith "cor_sup_d"
  in function
    | @((a b[Rouge] c) d e) when couleur e = NoirNoir ->
      @(a b[Noir] {aux @(c d[Rouge] {eclaircir e})})
    | @(a b c)              when couleur c = NoirNoir -> aux @(a b {eclaircir c})
    | a -> a

let assombrir = function
  | Vide            -> VideN
  | @(a b[Rouge] c) -> @(a b[Noir    ] c)
  | @(a b[Noir ] c) -> @(a b[NoirNoir] c)
  | _               -> invalid_arg "assombrir"

let rec minimum = function
  | @({Vide} r _) -> (rx, rv)
  | @(g _ _)      -> minimum g
  | _             -> failwith "minimum"

let rec supprimer_minimum = function
  | @({Vide} _[Rouge] d) -> d
  | @({Vide} _[Noir ] d) -> assombrir d
  | @(a b c)             -> cor_sup_g @({supprimer_minimum a} b c)
  | _                    -> failwith "supprimer_minimum"

let fusionner rc = function
  | (Vide, a)  | (a, Vide) -> if rc = Noir then assombrir a else a
  | (g, d) ->
      let (rx, rv) = minimum d in
      cor_sup_d @(g r {supprimer_minimum d})

let supprimer nx a =
  let rec aux = function
    | VideN                 -> invalid_arg "arbre rouge-noir incorrect"
    | Vide                  -> Vide
    | @(g r d) when nx < rx -> cor_sup_g @({aux g} r d)
    | @(g r d) when rx < nx -> cor_sup_d @(g r {aux d})
    | @(g _[c] d)           -> fusionner c (g, d)
  in match aux a with
    | VideN | Vide -> Vide
    | @(g r d)     -> @(g r[Noir] d)

(**********************************************************************)
