(** Tests al�toires pour arbres binaires de recherche *)

open Printf
open Arbres_naifs

let random_seed = try int_of_string (Sys.getenv "RANDOM_SEED") with _ -> 18
let max_int = try int_of_string (Sys.getenv "MAX_INT") with _ -> max_int
let nb_noeuds = try int_of_string (Sys.getenv "NB_NOEUDS") with _ -> 65536
let nb_work = try int_of_string (Sys.getenv "NB_WORK") with _ -> 64
let _ = try Gc.set { (Gc.get()) with Gc.verbose = (int_of_string (Sys.getenv "GC_VERBOSE")) } with _ -> ()

let chrono f a =
  let debut = Sys.time () in
  let res = f a in
  (Sys.time () -. debut, res)

let make () =
  let res = ref vide in
  for i = 1 to nb_noeuds do
    res := inserer (Random.int max_int) 0 !res;
  done; !res

let destroy arbre =
  let debut = Sys.time () in
  let res = ref arbre in
  for i = 0 to max_int-1 do
    res := supprimer i !res;
  done;
  let t = Sys.time () -. debut in (t, verifier !res)

let supp_random arbre =
  let debut = Sys.time () in
  let res = ref arbre in
  for i = 1 to nb_work do res := supprimer (Random.int max_int) !res done;
  let duree = Sys.time () -.debut in
  (duree, verifier !res)

let ins_random arbre =
  let debut = Sys.time () in
  let res = ref arbre in
  for i = 1 to nb_work do res := inserer (-(Random.int max_int)) 0 !res done;
  let duree = Sys.time () -.debut in
  (duree, verifier !res)


let () =  
  Random.init random_seed;
  printf "================================================================================\n";
  printf "Test al�toire de Arbres_naifs\n";
  printf "nb_noeuds=%d, nb_work=%d, random_seed=%d, max_int=%d\n" nb_noeuds nb_work random_seed max_int;
  printf "Construction... "; flush stdout;
  let (t, arbre) = chrono make () in
  printf "OK (%.2f s)\n" t;
  printf "V�ification... "; flush stdout;
  let (t,(n,h)) = chrono verifier arbre in
  printf "OK : %d noeuds, hauteur=%d (%.2f s)\n" n h t;
  printf "--------------------------------------------------------------------------------\n";

  printf "Test de suppression... "; flush stdout;
  let (t,(n,h)) = supp_random arbre in
  printf "OK : %d noeuds, hauteur=%d (%.2f s)\n" n h t;

  printf "Test d'insertion... "; flush stdout;
  let (t,(n,h)) = ins_random arbre in
  printf "OK : %d noeuds, hauteur=%d (%.2f s)\n" n h t;

  printf "Destruction... "; flush stdout;
  let (t,(n,h)) = destroy arbre in
  printf "OK : %d noeuds, hauteur=%d (%.2f s)\n" n h t;
  printf "================================================================================\n"
